@@ -37,9 +37,8 @@ use Date::ICal;
 use Set::Infinite::Element_Inf;
 
 use overload
-	'0+' => \&Date::ICal::epoch,
+	'0+' => sub { $_[0]->{epoch} },   # \&Date::ICal::epoch,
 	'<=>' => \&spaceship,
-	'cmp' => \&cmp,
 	'-' => \&sub,
 	'+' => \&add,
 	'/' => \&div,
@@ -55,10 +54,11 @@ sub quantizer {
 
 sub add {
 	my ($tmp1, $tmp2) = @_;
+	# print "+";
 	# print " [ical:add:", $tmp1, " + ", ref($tmp2), "->", $tmp2, "] ";
 
 	# unless (ref($tmp2) and $tmp2->isa(__PACKAGE__)) { $tmp2 = __PACKAGE__->new($tmp2); }
-	return __PACKAGE__->new( $tmp1->epoch + $tmp2 );
+	return __PACKAGE__->new( $tmp1->{epoch} + $tmp2 );
 
 	# return Date::ICal::add($tmp1, $tmp2);
 }
@@ -67,9 +67,10 @@ sub sub {
 	my ($tmp1, $tmp2, $inverted) = @_;
 	# print " [ical:sub:", $tmp1, " - ", ref($tmp2), "->", $tmp2, "] ";
 	# print " [duration:", $tmp1->epoch, "] ";
+	# print "-";
 
-	$tmp1 = $tmp1->epoch if ref($tmp1);
-	$tmp2 = $tmp2->epoch if ref($tmp2);
+	$tmp1 = $tmp1->{epoch} if ref($tmp1);
+	$tmp2 = $tmp2->{epoch} if ref($tmp2);
 
 	return $tmp2 - $tmp1 if $inverted;
 	return $tmp1 - $tmp2;
@@ -81,6 +82,7 @@ sub sub {
 
 sub div {
 	my ($tmp1, $tmp2) = @_;
+	# print "/";
 	return $tmp1 / $tmp2;
 }
 
@@ -88,69 +90,110 @@ sub spaceship {
 	my ($tmp1, $tmp2, $inverted) = @_;
 	return 1 unless defined($tmp2);
 
+	# print "=";
 	# print " [ical:cmp:", $tmp1, "<=>", ref($tmp2), "->", $tmp2, "] ";
 
 	# ???
-	return 1 if Set::Infinite::Element_Inf::is_null($tmp2);
+	# return 1 if Set::Infinite::Element_Inf::is_null($tmp2);
 
-	unless ( ref($tmp2) ) {   # and $tmp2->isa(__PACKAGE__) ) {
+	unless ( ref($tmp2) ) {                                   # and $tmp2->isa(__PACKAGE__) ) {
 		# print " [ical:cmp:new] ";
-		$tmp2 = __PACKAGE__->new($tmp2) ;
-	}
-	elsif ($tmp2->isa('Set::Infinite::Element_Inf')) {
-		# print " [ical:cmp:inf] ";
+		# print "N $tmp2 ";
+
 		if ($inverted) {
-			return Set::Infinite::Element_Inf::cmp($tmp2, $tmp1);
+			return $tmp2 <=> $tmp1->{epoch};
 		}
-		return Set::Infinite::Element_Inf::cmp($tmp1, $tmp2);
+		return $tmp1->{epoch} <=> $tmp2;
+
+		# $tmp2 = __PACKAGE__->new($tmp2) ;
 	}
+	elsif ( ref($tmp2) eq 'Set::Infinite::Element_Inf' ) {    # $tmp2->isa('Set::Infinite::Element_Inf')) {
+		# print " [ical:cmp:inf] ";
+		# print "I";
+		if ($inverted) {
+			return Set::Infinite::Element_Inf::spaceship($tmp2, $tmp1);
+		}
+		return Set::Infinite::Element_Inf::spaceship($tmp1, $tmp2);
+	}
+
+	# print "C";
+
+	# most frequent case
+
+	# $tmp1->{epoch} = $tmp1->epoch unless $tmp1->{epoch};
+	# $tmp2->{epoch} = $tmp2->epoch unless $tmp2->{epoch};
 
 	if ($inverted) {
-		return $tmp2->compare($tmp1);
+		return $tmp2->{epoch} <=> $tmp1->{epoch};
 	}
-	return $tmp1->compare($tmp2);
-}
-
-sub cmp {
-	return spaceship @_;
+	return $tmp1->{epoch} <=> $tmp2->{epoch};
 }
 
 sub as_string {
 	my ($self) = shift;
-	return $self->ical;
+	$self->{string} = $self->ical unless $self->{string};
+	return $self->{string};
 }
 
+our %new_cache = ();
+
 sub new {
 	my ($self) = shift;
 
+	# print "N";
 	# print " [ical:new:", join(';', @_) , "] ";
 
-	if ( Set::Infinite::Element_Inf->is_null($_[0]) ) {
+	if ( ($#_ < 0) or Set::Infinite::Element_Inf->is_null($_[0]) ) {
 		# print " [ical:new:null] ";
 		return Set::Infinite::Element_Inf->null ;
 	}
+	elsif (ref($_[0])) {
+		if ($_[0]->isa('Set::Infinite::Element_Inf')) {
+			# print " [ical:new:inf] ";
+			return $_[0];
+		}
+		else {
 
-	if (ref($_[0]) and $_[0]->isa('Set::Infinite::Element_Inf')) {
-		# print " [ical:new:inf] ";
-		return $_[0];
+		}
+	}
+	# get it from " $new_cache "
+	elsif (exists $new_cache{$_[0]}) {
+		# print "V";
+		$self = $new_cache{$_[0]};
+		return $self;
 	}
 
-	# print " [new...";
-	if ($#_ == 0) {
+	elsif ($#_ == 0) {
 		# epoch or ical mode?
 		my $value = $_[0];
 		if ($value =~ /^[12]\d{7}/) {
+			# print "1";
+			# must be ical format
 			$self = Date::ICal->new( ical => $_[0] );
+			bless $self, __PACKAGE__;
+			$self->{string} = $_[0];    # cache string
+			$self->{epoch} = $self->epoch;  # cache epoch
+			$new_cache{$_[0]} = $self;  # cache object
+			return $self;
 		}
 		else {
-			# NOT 19971024
+			# NOT 19971024 -- must be "epoch"
+			# print "2";
+			# most frequent case: use " $new_cache " to optimize
+
 			$self = Date::ICal->new( epoch => $_[0] );
+			bless $self, __PACKAGE__;
+			$self->{epoch} = $_[0];     # cache epoch
+			$new_cache{$_[0]} = $self;  # cache object
+			return $self;
 		}
 	}
 	else {
+		# print "3";
 		$self = Date::ICal->new( @_ );
 	}
 	bless $self, __PACKAGE__;
+	$self->{epoch} = $self->epoch;      # cache epoch
 	# print " [ical:new:", $self , "] ";
 	return $self;
 }
